@@ -52,6 +52,7 @@ static int
 				Dwarf_Signed * fde_element_count,
 				Dwarf_Small * section_ptr,
 				Dwarf_Unsigned section_length,
+				Dwarf_Unsigned section_index,
 				Dwarf_Unsigned cie_id_value,
 				int use_gnu_cie_calc,
 				Dwarf_Error * error);
@@ -784,8 +785,6 @@ get_augmentation_string(Dwarf_Debug dbg,
     READ_AREA_LENGTH(dbg, length, Dwarf_Unsigned,
 		     cie_ptr, local_length_size, local_extension_size);
 
-
-
     /* Read the Cie Id field. */
     READ_UNALIGNED(dbg, cie_id, Dwarf_Unsigned,
 		   cie_ptr, local_length_size);
@@ -803,7 +802,8 @@ get_augmentation_string(Dwarf_Debug dbg,
     /* Read the version. */
     version = *(Dwarf_Small *) cie_ptr;
     cie_ptr++;
-    if (version != DW_CIE_VERSION) {
+    if (version != DW_CIE_VERSION &&
+	version != DW_CIE_VERSION3) {
 	_dwarf_error(dbg, error, DW_DLE_FRAME_VERSION_BAD);
 	return (DW_DLV_ERROR);
     }
@@ -866,6 +866,7 @@ dwarf_get_fde_list_eh(Dwarf_Debug dbg,
 				      fde_element_count,
 				      dbg->de_debug_frame_eh_gnu,
 				      dbg->de_debug_frame_size_eh_gnu,
+				      dbg->de_debug_frame_eh_gnu_index,
 				      /* cie_id_value */ 0,
 				      /* use_gnu_cie_calc= */ 1,
 				      error);
@@ -907,9 +908,11 @@ dwarf_get_fde_list(Dwarf_Debug dbg,
 				      fde_element_count,
 				      dbg->de_debug_frame,
 				      dbg->de_debug_frame_size,
+				      dbg->de_debug_frame_index,
 				      DW_CIE_ID,
 				      /* use_gnu_cie_calc= */ 0,
 				      error);
+
     return res;
 }
 
@@ -921,8 +924,10 @@ __dwarf_get_fde_list_internal(Dwarf_Debug dbg,
 			      Dwarf_Signed * fde_element_count,
 			      Dwarf_Small * section_ptr,
 			      Dwarf_Unsigned section_length,
+			      Dwarf_Unsigned section_index,
 			      Dwarf_Unsigned cie_id_value,
-			      int use_gnu_cie_calc, Dwarf_Error * error)
+			      int use_gnu_cie_calc, 
+			      Dwarf_Error * error)
 {
     /* Scans the debug_frame section. */
     Dwarf_Small *frame_ptr = 0;
@@ -1050,7 +1055,8 @@ __dwarf_get_fde_list_internal(Dwarf_Debug dbg,
 	       spec, section 6.4.1 */
 	    version = *(Dwarf_Small *) frame_ptr;
 	    frame_ptr++;
-	    if (version != DW_CIE_VERSION) {
+	    if (version != DW_CIE_VERSION &&
+		version != DW_CIE_VERSION3) {
 		_dwarf_error(dbg, error, DW_DLE_FRAME_VERSION_BAD);
 		return (DW_DLV_ERROR);
 	    }
@@ -1161,6 +1167,7 @@ __dwarf_get_fde_list_internal(Dwarf_Debug dbg,
 		return (DW_DLV_ERROR);
 	    }
 
+	    new_cie->ci_cie_version_number = version;
 	    new_cie->ci_initial_table = NULL;
 	    new_cie->ci_length = (Dwarf_Word) length;
 	    new_cie->ci_length_size = local_length_size;
@@ -1297,6 +1304,9 @@ __dwarf_get_fde_list_internal(Dwarf_Debug dbg,
 	    new_fde->fd_dbg = dbg;
 	    new_fde->fd_offset_into_exception_tables =
 		offset_into_exception_tables;
+	    new_fde->fd_section_ptr = section_ptr;
+	    new_fde->fd_section_length = section_length;
+	    new_fde->fd_section_index = section_index;
 
 	    fde_count++;
 	    if (head_fde_ptr == NULL)
@@ -1556,6 +1566,10 @@ dwarf_get_fde_for_die(Dwarf_Debug dbg,
     new_fde->fd_dbg = dbg;
     new_fde->fd_offset_into_exception_tables =
 	offset_into_exception_tables;
+    new_fde->fd_section_ptr = dbg->de_debug_frame;
+    new_fde->fd_section_length = dbg->de_debug_frame_size;
+    new_fde->fd_section_index = dbg->de_debug_frame_index;
+
 
     /* now read the cie corresponding to the fde */
     cie_ptr = (dbg->de_debug_frame + signed_offset);
@@ -1581,7 +1595,8 @@ dwarf_get_fde_for_die(Dwarf_Debug dbg,
 
 	version = *(Dwarf_Small *) cie_ptr;
 	cie_ptr++;
-	if (version != DW_CIE_VERSION) {
+	if (version != DW_CIE_VERSION &&
+		version != DW_CIE_VERSION3) {
 	    _dwarf_error(dbg, error, DW_DLE_FRAME_VERSION_BAD);
 	    return (DW_DLV_ERROR);
 	}
@@ -1670,6 +1685,7 @@ dwarf_get_fde_for_die(Dwarf_Debug dbg,
 	    return (DW_DLV_ERROR);
 	}
 
+        new_cie->ci_cie_version_number = version;
 	new_cie->ci_initial_table = NULL;
 	new_cie->ci_length = (Dwarf_Word) length;
 	new_cie->ci_length_size = cie_local_length_size;
@@ -1704,7 +1720,6 @@ dwarf_get_fde_range(Dwarf_Fde fde,
 		    Dwarf_Signed * cie_index,
 		    Dwarf_Off * fde_offset, Dwarf_Error * error)
 {
-    int res;
     Dwarf_Debug dbg;
 
     if (fde == NULL) {
@@ -1718,14 +1733,12 @@ dwarf_get_fde_range(Dwarf_Fde fde,
 	return (DW_DLV_ERROR);
     }
 
-    res =
-        _dwarf_load_section(dbg,
-			    dbg->de_debug_frame_index,
-			    &dbg->de_debug_frame,
-			    error);
-    if (res != DW_DLV_OK) {
-        return res;
-    }
+	
+    /* We have always already done the section load here, so
+       no need to load the section.
+       We did the section load in order to create the Dwarf_Fde pointer
+       passed in here. */
+
 
     if (low_pc != NULL)
 	*low_pc = fde->fd_initial_location;
@@ -1740,7 +1753,7 @@ dwarf_get_fde_range(Dwarf_Fde fde,
     if (cie_index != NULL)
 	*cie_index = fde->fd_cie_index;
     if (fde_offset != NULL)
-	*fde_offset = fde->fd_fde_start - dbg->de_debug_frame;
+	*fde_offset = fde->fd_fde_start - fde->fd_section_ptr;
 
     return DW_DLV_OK;
 }
@@ -1767,7 +1780,7 @@ dwarf_get_fde_exception_info(Dwarf_Fde fde,
 int
 dwarf_get_cie_info(Dwarf_Cie cie,
 		   Dwarf_Unsigned * bytes_in_cie,
-		   Dwarf_Small * version,
+		   Dwarf_Small * ptr_to_version,
 		   char **augmenter,
 		   Dwarf_Unsigned * code_alignment_factor,
 		   Dwarf_Signed * data_alignment_factor,
@@ -1789,8 +1802,8 @@ dwarf_get_cie_info(Dwarf_Cie cie,
 	return (DW_DLV_ERROR);
     }
 
-    if (version != NULL)
-	*version = DW_CIE_VERSION;
+    if (ptr_to_version != NULL)
+	*ptr_to_version = cie->ci_cie_version_number;
     if (augmenter != NULL)
 	*augmenter = cie->ci_augmentation;
     if (code_alignment_factor != NULL)
